@@ -30,7 +30,8 @@
 #include "libglnx/libglnx.h"
 
 #include "flatpak-builtins.h"
-#include "flatpak-utils.h"
+#include "flatpak-utils-private.h"
+#include "parse-datetime.h"
 
 static char *opt_subject;
 static char *opt_body;
@@ -38,6 +39,8 @@ static char *opt_arch;
 static gboolean opt_runtime;
 static gboolean opt_update_appstream;
 static gboolean opt_no_update_summary;
+static gboolean opt_disable_fsync;
+static gboolean opt_disable_sandbox = FALSE;
 static char **opt_gpg_key_ids;
 static char **opt_exclude;
 static char **opt_include;
@@ -45,9 +48,8 @@ static char *opt_gpg_homedir;
 static char *opt_files;
 static char *opt_metadata;
 static char *opt_timestamp = NULL;
-#ifdef FLATPAK_ENABLE_P2P
+static char *opt_endoflife;
 static char *opt_collection_id = NULL;
-#endif  /* FLATPAK_ENABLE_P2P */
 
 static GOptionEntry options[] = {
   { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, N_("One line subject"), N_("SUBJECT") },
@@ -62,10 +64,11 @@ static GOptionEntry options[] = {
   { "exclude", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_exclude, N_("Files to exclude"), N_("PATTERN") },
   { "include", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_include, N_("Excluded files to include"), N_("PATTERN") },
   { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, N_("GPG Homedir to use when looking for keyrings"), N_("HOMEDIR") },
-  { "timestamp", 0, 0, G_OPTION_ARG_STRING, &opt_timestamp, N_("Override the timestamp of the commit"), N_("ISO-8601-TIMESTAMP") },
-#ifdef FLATPAK_ENABLE_P2P
+  { "end-of-life", 0, 0, G_OPTION_ARG_STRING, &opt_endoflife, N_("Mark build as end-of-life"), N_("REASON") },
+  { "timestamp", 0, 0, G_OPTION_ARG_STRING, &opt_timestamp, N_("Override the timestamp of the commit"), N_("TIMESTAMP") },
   { "collection-id", 0, 0, G_OPTION_ARG_STRING, &opt_collection_id, N_("Collection ID"), "COLLECTION-ID" },
-#endif  /* FLATPAK_ENABLE_P2P */
+  { "disable-fsync", 0, 0, G_OPTION_ARG_NONE, &opt_disable_fsync, "Do not invoke fsync()", NULL },
+  { "disable-sandbox", 0, 0, G_OPTION_ARG_NONE, &opt_disable_sandbox, "Do not sandbox icon validator", NULL },
 
   { NULL }
 };
@@ -181,7 +184,7 @@ commit_filter (OstreeRepo *repo,
    * case. So, we canonicalize the mode to writable only by the user,
    * readable to all, and executable for all for directories and
    * files that the user can execute.
-  */
+   */
   mode = g_file_info_get_attribute_uint32 (file_info, "unix::mode");
   if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY)
     mode = 0755 | S_IFDIR;
@@ -267,30 +270,147 @@ find_file_in_tree (GFile *base, const char *filename)
   if (!enumerator)
     return FALSE;
 
-  do {
-    g_autoptr(GFileInfo) info = g_file_enumerator_next_file (enumerator, NULL, NULL);
-    GFileType type;
-    const char *name;
+  do
+    {
+      g_autoptr(GFileInfo) info = g_file_enumerator_next_file (enumerator, NULL, NULL);
+      GFileType type;
+      const char *name;
 
-    if (!info)
-      return FALSE;
+      if (!info)
+        return FALSE;
 
-    type = g_file_info_get_file_type (info);
-    name = g_file_info_get_name (info);
+      type = g_file_info_get_file_type (info);
+      name = g_file_info_get_name (info);
 
-    if (type == G_FILE_TYPE_REGULAR && strcmp (name, filename) == 0)
-      return TRUE;
-    else if (type == G_FILE_TYPE_DIRECTORY)
-      {
-        g_autoptr(GFile) dir = g_file_get_child (base, name);
-        if (find_file_in_tree (dir, filename))
-          return TRUE;
-      }
-  } while (1);
+      if (type == G_FILE_TYPE_REGULAR && strcmp (name, filename) == 0)
+        return TRUE;
+      else if (type == G_FILE_TYPE_DIRECTORY)
+        {
+          g_autoptr(GFile) dir = g_file_get_child (base, name);
+          if (find_file_in_tree (dir, filename))
+            return TRUE;
+        }
+    }
+  while (1);
 
   return FALSE;
 }
 
+typedef gboolean (* VisitFileFunc) (GFile *file, GError **error);
+
+static gboolean
+visit_files_in_tree (GFile *base,
+                     VisitFileFunc visit_file,
+                     gpointer data)
+{
+  g_autoptr(GFileEnumerator) enumerator = NULL;
+  GError **error = data;
+
+  enumerator = g_file_enumerate_children (base,
+                                          G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+                                          G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                          G_FILE_QUERY_INFO_NONE,
+                                          NULL,
+                                          NULL);
+  if (!enumerator)
+    return FALSE;
+
+  do
+    {
+      GFileInfo *info;
+      GFile *child;
+      GFileType type;
+
+      if (!g_file_enumerator_iterate (enumerator, &info, &child, NULL, error))
+        return FALSE;
+
+      if (!info)
+        return TRUE;
+
+      type = g_file_info_get_file_type (info);
+
+      if (type == G_FILE_TYPE_REGULAR)
+        {
+          if (!visit_file (child, data))
+            return FALSE;
+        }
+      else if (type == G_FILE_TYPE_DIRECTORY)
+        {
+          if (!visit_files_in_tree (child, visit_file, data))
+            return FALSE;
+        }
+    }
+  while (1);
+
+  return TRUE;
+}
+
+G_GNUC_NULL_TERMINATED
+static void
+add_args (GPtrArray *argv_array, ...)
+{
+  va_list args;
+  const char *arg;
+
+  va_start (args, argv_array);
+  while ((arg = va_arg (args, const gchar *)))
+    g_ptr_array_add (argv_array, g_strdup (arg));
+  va_end (args);
+}
+
+static gboolean
+validate_icon_file (GFile *file, GError **error)
+{
+  g_autoptr(GPtrArray) args = NULL;
+  const char *name;
+  int status;
+  g_autofree char *err = NULL;
+  const char *validate_icon = LIBEXECDIR "/flatpak-validate-icon";
+
+  name = flatpak_file_get_path_cached (file);
+
+  if (g_getenv ("FLATPAK_VALIDATE_ICON"))
+    validate_icon = g_getenv ("FLATPAK_VALIDATE_ICON");
+
+  args = g_ptr_array_new_with_free_func (g_free);
+
+#ifndef DISABLE_SANDBOXED_TRIGGERS
+  if (!opt_disable_sandbox)
+    add_args (args, validate_icon, "--sandbox", "512", "512", name, NULL);
+  else
+#endif
+    add_args (args, validate_icon, "512", "512", name, NULL);
+
+  g_ptr_array_add (args, NULL);
+
+  if (!g_spawn_sync (NULL, (char **)args->pdata, NULL, 0, NULL, NULL, NULL, &err, &status, error))
+    {
+      g_debug ("Icon validation: %s", (*error)->message);
+      return FALSE;
+    }
+
+  if (!g_spawn_check_exit_status (status, NULL))
+    {
+      g_debug ("Icon validation: %s", err);
+      return flatpak_fail (error, "%s is not a valid icon: %s", name, err);
+    }
+
+  return TRUE;
+}
+
+static gboolean
+validate_exported_icons (GFile      *export,
+                         const char *app_id,
+                         GError    **error)
+{
+  g_autoptr(GFile) icondir = NULL;
+
+  icondir = g_file_resolve_relative_path (export, "share/icons/hicolor");
+  visit_files_in_tree (icondir, validate_icon_file, error);
+
+  return !*error;
+}
+
 static GFile *
 convert_app_absolute_path (const char *path, GFile *files)
 {
@@ -310,15 +430,16 @@ convert_app_absolute_path (const char *path, GFile *files)
 }
 
 static gboolean
-validate_desktop_file (GFile *desktop_file,
-                       GFile *export,
-                       GFile *files,
+validate_desktop_file (GFile      *desktop_file,
+                       GFile      *export,
+                       GFile      *files,
                        const char *app_id,
-                       char **icon,
-                       gboolean *activatable,
-                       GError **error)
+                       char      **icon,
+                       gboolean   *activatable,
+                       GError    **error)
 {
   g_autofree char *path = g_file_get_path (desktop_file);
+
   g_autoptr(GSubprocess) subprocess = NULL;
   g_autofree char *stdout_buf = NULL;
   g_autofree char *stderr_buf = NULL;
@@ -394,9 +515,9 @@ check_refs:
 
 static gboolean
 validate_icon (const char *icon,
-               GFile *export,
+               GFile      *export,
                const char *app_id,
-               GError **error)
+               GError    **error)
 {
   g_autoptr(GFile) icondir = NULL;
   g_autofree char *png = NULL;
@@ -416,13 +537,14 @@ validate_icon (const char *icon,
 }
 
 static gboolean
-validate_service_file (GFile *service_file,
-                       gboolean activatable,
-                       GFile *files,
+validate_service_file (GFile      *service_file,
+                       gboolean    activatable,
+                       GFile      *files,
                        const char *app_id,
-                       GError **error)
+                       GError    **error)
 {
   g_autofree char *path = g_file_get_path (service_file);
+
   g_autoptr(GKeyFile) key_file = NULL;
   g_autofree char *name = NULL;
   g_autofree char *command = NULL;
@@ -481,6 +603,7 @@ static gboolean
 validate_exports (GFile *export, GFile *files, const char *app_id, GError **error)
 {
   g_autofree char *desktop_path = NULL;
+
   g_autoptr(GFile) desktop_file = NULL;
   g_autofree char *service_path = NULL;
   g_autoptr(GFile) service_file = NULL;
@@ -496,6 +619,9 @@ validate_exports (GFile *export, GFile *files, const char *app_id, GError **erro
   if (!validate_icon (icon, export, app_id, error))
     return FALSE;
 
+  if (!validate_exported_icons (export, app_id, error))
+    return FALSE;
+
   service_path = g_strconcat ("share/dbus-1/services/", app_id, ".service", NULL);
   service_file = g_file_resolve_relative_path (export, service_path);
 
@@ -655,8 +781,8 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
   g_autoptr(GVariant) metadata_dict_v = NULL;
   gboolean is_runtime = FALSE;
   gboolean is_extension = FALSE;
-  guint64 installed_size = 0,download_size = 0;
-  GTimeVal ts;
+  guint64 installed_size = 0, download_size = 0;
+  struct timespec ts;
   const char *collection_id;
 
   context = g_option_context_new (_("LOCATION DIRECTORY [BRANCH] - Create a repository from a build directory"));
@@ -685,14 +811,12 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
   else
     branch = "master";
 
-#ifdef FLATPAK_ENABLE_P2P
   if (opt_collection_id != NULL &&
       !ostree_validate_collection_id (opt_collection_id, &my_error))
     {
       flatpak_fail (error, _("‘%s’ is not a valid collection ID: %s"), opt_collection_id, my_error->message);
       goto out;
     }
-#endif  /* FLATPAK_ENABLE_P2P */
 
   if (!flatpak_is_valid_branch (branch, &my_error))
     {
@@ -772,7 +896,7 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
     body = g_strdup_printf ("Name: %s\n"
                             "Arch: %s\n"
                             "Branch: %s\n"
-                            "Built with: "PACKAGE_STRING"\n",
+                            "Built with: "PACKAGE_STRING "\n",
                             id, arch, branch);
 
   full_branch = g_strconcat ((opt_runtime || is_runtime) ? "runtime/" : "app/", id, "/", arch, "/", branch, NULL);
@@ -789,7 +913,6 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
       if (!ostree_repo_resolve_rev (repo, full_branch, TRUE, &parent, error))
         goto out;
 
-#ifdef FLATPAK_ENABLE_P2P
       if (opt_collection_id != NULL &&
           g_strcmp0 (ostree_repo_get_collection_id (repo), opt_collection_id) != 0)
         {
@@ -797,27 +920,23 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
                         opt_collection_id, ostree_repo_get_collection_id (repo));
           goto out;
         }
-#endif  /* FLATPAK_ENABLE_P2P */
     }
   else
     {
-#ifdef FLATPAK_ENABLE_P2P
       if (opt_collection_id != NULL &&
           !ostree_repo_set_collection_id (repo, opt_collection_id, error))
         goto out;
-#endif  /* FLATPAK_ENABLE_P2P */
       if (!ostree_repo_create (repo, OSTREE_REPO_MODE_ARCHIVE_Z2, cancellable, error))
         goto out;
     }
 
+  if (opt_disable_fsync)
+    ostree_repo_set_disable_fsync (repo, TRUE);
+
   /* Get the canonical collection ID which we’ll use for the commit. This might
    * be %NULL if the existing repo doesn’t have one and none was specified on
    * the command line. */
-#ifdef FLATPAK_ENABLE_P2P
   collection_id = ostree_repo_get_collection_id (repo);
-#else  /* if !FLATPAK_ENABLE_P2P */
-  collection_id = NULL;
-#endif  /* !FLATPAK_ENABLE_P2P */
 
   if (!ostree_repo_prepare_transaction (repo, NULL, cancellable, error))
     goto out;
@@ -883,8 +1002,7 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
     goto out;
 
   /* Binding information. xa.ref is deprecated in favour of the OSTree keys, but
-   * keep it around for backwards compatibility. Write the bindings even if
-   * we’re compiled without P2P support, since other flatpak builds might be. */
+   * keep it around for backwards compatibility. */
   g_variant_dict_insert_value (&metadata_dict, "ostree.collection-binding",
                                g_variant_new_string ((collection_id != NULL) ? collection_id : ""));
   g_variant_dict_insert_value (&metadata_dict, "ostree.ref-binding",
@@ -895,17 +1013,17 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
   g_variant_dict_insert_value (&metadata_dict, "xa.installed-size", g_variant_new_uint64 (GUINT64_TO_BE (installed_size)));
   g_variant_dict_insert_value (&metadata_dict, "xa.download-size", g_variant_new_uint64 (GUINT64_TO_BE (download_size)));
 
+  if (opt_endoflife && *opt_endoflife)
+    g_variant_dict_insert_value (&metadata_dict, OSTREE_COMMIT_META_KEY_ENDOFLIFE,
+                                 g_variant_new_string (opt_endoflife));
+
   metadata_dict_v = g_variant_ref_sink (g_variant_dict_end (&metadata_dict));
 
   /* The timestamp is used for the commit metadata and AppStream data */
   if (opt_timestamp != NULL)
     {
-      if (!g_time_val_from_iso8601 (opt_timestamp, &ts))
-        {
-          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                       "Could not parse '%s'", opt_timestamp);
-          goto out;
-        }
+      if (!parse_datetime (&ts, opt_timestamp, NULL))
+        return flatpak_fail (error, _("Could not parse '%s'"), opt_timestamp);
     }
 
   if (opt_timestamp == NULL)
@@ -943,14 +1061,12 @@ flatpak_builtin_build_export (int argc, char **argv, GCancellable *cancellable,
         }
     }
 
-#ifdef FLATPAK_ENABLE_P2P
   if (collection_id != NULL)
     {
       OstreeCollectionRef ref = { (char *) collection_id, full_branch };
       ostree_repo_transaction_set_collection_ref (repo, &ref, commit_checksum);
     }
   else
-#endif  /* FLATPAK_ENABLE_P2P */
     {
       ostree_repo_transaction_set_ref (repo, NULL, full_branch, commit_checksum);
     }
