@@ -67,6 +67,7 @@ gboolean flatpak_run_in_transient_unit (const char *app_id,
 #define FLATPAK_METADATA_KEY_EXTRA_ARGS "extra-args"
 #define FLATPAK_METADATA_KEY_SANDBOX "sandbox"
 #define FLATPAK_METADATA_KEY_BUILD "build"
+#define FLATPAK_METADATA_KEY_DEVEL "devel"
 
 #define FLATPAK_METADATA_GROUP_SESSION_BUS_POLICY "Session Bus Policy"
 #define FLATPAK_METADATA_GROUP_SYSTEM_BUS_POLICY "System Bus Policy"
@@ -103,6 +104,9 @@ gboolean flatpak_run_in_transient_unit (const char *app_id,
 #define FLATPAK_METADATA_KEY_REF "ref"
 #define FLATPAK_METADATA_KEY_TAG "tag"
 
+#define FLATPAK_METADATA_GROUP_DCONF "X-DConf"
+#define FLATPAK_METADATA_KEY_DCONF_PATHS "paths"
+
 gboolean  flatpak_run_add_extension_args (FlatpakBwrap *bwrap,
                                           GKeyFile     *metakey,
                                           const char   *full_ref,
@@ -156,6 +160,7 @@ gboolean flatpak_run_add_app_info_args (FlatpakBwrap   *bwrap,
                                         FlatpakContext *cmdline_context,
                                         gboolean        sandbox,
                                         gboolean        build,
+                                        gboolean        devel,
                                         char          **app_info_path_out,
                                         char          **host_instance_id_host_dir_out,
                                         GError        **error);
@@ -167,9 +172,11 @@ gboolean flatpak_run_app (const char     *app_ref,
                           const char     *custom_runtime_version,
                           const char     *custom_runtime_commit,
                           FlatpakRunFlags flags,
+                          const char     *cwd,
                           const char     *custom_command,
                           char           *args[],
                           int             n_args,
+                          char          **instance_dir_out,
                           GCancellable   *cancellable,
                           GError        **error);
 
