@@ -26,6 +26,13 @@ typedef enum {
   FLATPAK_SPAWN_FLAGS_LATEST_VERSION = 1 << 1,
   FLATPAK_SPAWN_FLAGS_SANDBOX = 1 << 2,
   FLATPAK_SPAWN_FLAGS_NO_NETWORK = 1 << 3,
+  FLATPAK_SPAWN_FLAGS_WATCH_BUS = 1 << 4,
 } FlatpakSpawnFlags;
 
+#define FLATPAK_SPAWN_FLAGS_ALL (FLATPAK_SPAWN_FLAGS_CLEAR_ENV | \
+                                 FLATPAK_SPAWN_FLAGS_LATEST_VERSION | \
+                                 FLATPAK_SPAWN_FLAGS_SANDBOX | \
+                                 FLATPAK_SPAWN_FLAGS_NO_NETWORK | \
+                                 FLATPAK_SPAWN_FLAGS_WATCH_BUS)
+
 #endif /* __FLATPAK_PORTAL_H__ */
